#include <libguile.h>

typedef SCM(*jit_func_t)();

#include "llvm/Module.h"
#include "llvm/DerivedTypes.h"
#include "llvm/Constants.h"
#include "llvm/Instructions.h"
#include "llvm/ModuleProvider.h"
#include "llvm/Analysis/Verifier.h"
#include "llvm/ExecutionEngine/JIT.h"
#include "llvm/ExecutionEngine/Interpreter.h"
#include "llvm/ExecutionEngine/GenericValue.h"
#include "llvm/Support/LLVMBuilder.h"
#include "llvm/PassManager.h"
#include "llvm/Transforms/Scalar.h"
#include <iostream>
#include <map>
#include <sstream>

using namespace llvm;
using namespace std;

class FunctionBuilder {
    Module *mModule;
    ExistingModuleProvider *mModuleProvider;
    FunctionPassManager *mPassManager;
    ExecutionEngine *mExecutionEngine;
    LLVMBuilder mBuilder;
    Function *mUnpackValue;
    Function *mPackValue;
    map<string, AllocaInst*> mBindingTable;
    
public:
    FunctionBuilder::FunctionBuilder(Module *lModule);
    
    Function *
    CreateFunction(const char *name, SCM arg_ls, SCM body);
    
    jit_func_t
    CompileJIT(const char* name, SCM args, SCM body);
    
    void
    ImportFunction(const char* name, int num_args);

    std::string dump() {
        ostringstream os;
        os << (*mModule);
        return os.str();
    }
protected:
    Value *CreateBody(SCM body);
};
