#include "coreJIT.h"
#include <iostream>

FunctionBuilder *gFunctionBuilder;

extern "C" {
    SCM jit_compile(SCM name, SCM args, SCM body);
    SCM jit_import(SCM name, SCM num_args);
    SCM llvm_to_string();
    
	void
    init_guileJIT() {
        gFunctionBuilder = new FunctionBuilder(new Module("guileJIT"));
        scm_c_define_gsubr("jit-compile", 3, 0, 0, (SCM(*)()) jit_compile);
        scm_c_define_gsubr("jit-import", 2, 0, 0, (SCM(*)()) jit_import);     
        scm_c_define_gsubr("llvm->string", 0, 0, 0, (SCM(*)()) llvm_to_string);
    }
    
    SCM jit_compile(SCM name, SCM args, SCM body) {
        int arg_len = scm_to_int(scm_length(args));
        char *cstr_name = scm_to_locale_string(name);
        
        jit_func_t func_ptr;
        
        try {
            func_ptr = gFunctionBuilder->CompileJIT(cstr_name, args, body);
        } catch (std::exception e) {
            scm_error(scm_from_locale_symbol("misc-error"), 
                      "jit-compile",
                      "(LLVM Error) unable to compile function",
                      SCM_BOOL_F, SCM_UNSPECIFIED);
            
            free(cstr_name);
            return SCM_UNSPECIFIED;
        }

        SCM ret_val = scm_c_define_gsubr(cstr_name, arg_len, 0, 0, func_ptr);
        free(cstr_name);
        return ret_val;
    }
    
    SCM jit_import(SCM name, SCM num_args) {
        char *cstr_name = scm_to_locale_string(name);
        gFunctionBuilder->ImportFunction(cstr_name, scm_to_int(num_args));
        free(cstr_name);
        return SCM_UNSPECIFIED;
    }
    
    SCM llvm_to_string() {
        std::string module_text(gFunctionBuilder->dump());
        return scm_from_locale_string(module_text.c_str());
    }
}
